#include <stdio.h>
#include <assert.h>
#include <getopt.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <netdb.h>   
#include <string.h>
#include <time.h>    

#include "eon.h"
#include "bool.h"
#include "config.h"

#ifndef NDEBUG
static bool verbose = true; /* == true iff verbose mode is selected */
#else
static bool verbose = false;
#endif

/* ---- Dump-mode flag ---- */
static bool dump = false;

/* ---- Do not make only one big group of repetitive dives ---- */
static bool flatten = false;

/* ---- Read-timeout in seconds ---- */
const unsigned int timeout = 60;

/* ---- Modi for different divecomputer models ---- */
typedef enum {
 EONMode,
 SolMode} DCMode;
static DCMode mode = EONMode;

/* ---- Program-names for switching to different modes ---- */
static char EONName[] = "eonconvert"; 
static char SolName[] = "solaconvert"; 

/* ---- Lenght of the personal info string ---- */
#define PERSINFOLEN 19

/* ---- 1 Bar == 100000 Pa ---- */
#define BAR2PA 100000

/* ---- 1ft == 0.3048 m ---- */
#define FEET2METER 0.3048

/* ---- 1m == 3.2808399 ft ---- */
#define METER2FEET 3.2808399

/* ---- Number of samples in one line in the <SAMPLE></SAMPLE> section ---- */
#define COLUMNS 6

/* ---- Maxiamal length of a FQN hostname ---- */
#define MAXDOMAINNAMELEN 256 

/* ---- Maximal length of a string holding local time ---- */
#define MAXTIMELEN 64

bool getFQDomain(char *domain, int len)
{
 struct hostent* he = (struct hostent*) NULL;
 
 /* ---- Get the hostname ---- */
 if (gethostname(domain, len - 1) == -1) {
 
  /* ---- Can't get hostname (show an error) ---- */
  domain[0] = '\0';
  return false;
 
 };
 
 /* ---- Use the resolver ---- */
 he = gethostbyname(domain);
 
 /* --- And check gethostbyname-call ---- */
 if (he == (struct hostent *) NULL) {
 
  /* ---- Can't get FQDN (show an error) ---- */
  domain[0] = '\0';
  return false;
 
 }                                                                    

 /* ---- Now copy result (always terminated with '\0' )---- */
 strncpy(domain, he -> h_name, (size_t) len - 1);
 domain[len - 1] = '\0';

 /* ---- Everthing is ok ---- */
 return true;
 
}

void PrintHelp(char* prgname)
{
#ifdef HAVE_GETOPTLONG
 /* ---- Print help message ---- */
 fprintf(stderr, "Usage: %s [OPTION(s)]\n", prgname);
 if (verbose) 
  fprintf(stderr, "$Id: eonconvert.c,v 1.40 2002/02/23 14:22:26 streit Exp $\n");
 fprintf(stderr, "-h --help                 Print a help text\n");
 fprintf(stderr, "-v --verbose              Explain what is done\n");
 fprintf(stderr, "-p --dump                 Print a hexdump of read data\n");
 fprintf(stderr, "-f --flatten              All dives into one group\n");
 fprintf(stderr, "-c --checksum             Check data (calculate a checksum)\n");
#else
 /* ---- Print help message ---- */
 fprintf(stderr, "Usage: %s [OPTION(s)]\n", prgname);
 if (verbose) 
  fprintf(stderr, "$Id: eonconvert.c,v 1.40 2002/02/23 14:22:26 streit Exp $\n");
 fprintf(stderr, "-h                        Print a help text\n");
 fprintf(stderr, "-v                        Explain what is done\n");
 fprintf(stderr, "-p                        Print a hexdump of read data\n");
 fprintf(stderr, "-f                        All dives into one group\n");
 fprintf(stderr, "-c                        Check data (calculate a checksum)\n");
#endif


 /* ---- Terminate program successfully ---- */
 exit(EXIT_SUCCESS);

}

void AlarmHandler(int dummy)
{
 /* ---- Tell an timeout and quit ---- */
 fprintf(stderr, "A timeout occured while reading profile data!\n");
 fprintf(stderr, "Better we quit.\n");
 exit(EXIT_FAILURE);

}

#define HEXBASE 16
void HexDumpBuffer(char buffer[BUFFSIZE])
{
 const char convert[HEXBASE] = "0123456789ABCDEF";
 const int cols = 32;
 int i;

 /* ---- Walk through whole buffer ---- */
 for (i = 0; i < BUFFSIZE; i++) {

  /* ---- Check if we need an offset ---- */
  if ((i % cols) == 0) fprintf(stderr, "%8.8X: ", i);

  /* ---- Print an entry ---- */
  fprintf(stderr, "%c%c", 
                  convert[(((unsigned long) buffer[i] / HEXBASE) % HEXBASE)],
                  convert[((unsigned long) buffer[i] % HEXBASE)]);

  /* ---- Check the type of seperator ---- */
  if ((i % cols) < (cols - 1))
   fprintf(stderr, ":"); 
  else 
   fprintf(stderr, "\n");

 }

 /* ---- Add a new line ---- */
 fprintf(stderr, "\n\n");

}

void ReadBuffer(char (*buffer)[BUFFSIZE])
{
#ifndef NDEBUG
 /* ---- If verbose report the installation of the alarm handler ---- */
 if(verbose) fprintf(stderr, "Installing the alarm handler!\n");
#endif

 /* ---- Set the alarm-handler ---- */
 if (signal(SIGALRM, AlarmHandler) == SIG_ERR) {

  /* ---- Give a warning and quit ---- */
  fprintf(stderr, "Can't install alarm handler!\n");
  fprintf(stderr, "Have to quit!\n");
  exit(EXIT_FAILURE); 

 }

 /* ---- Set the alarm ---- */
 (void) alarm(timeout);

#ifndef NDEBUG
 /* ---- If in verbose mode report "Start reading the profile" ---- */
 if (verbose) fprintf(stderr, "Start reading the buffer\n");
#endif

 /* ---- Read the data from stdin ---- */
 if (read(STDIN_FILENO, buffer, BUFFSIZE) != BUFFSIZE) {

  /* ---- We can't read the right number of bytes ---- */
  fprintf(stderr, "Cannot read the profile completely! Have to quit\n");
  exit(EXIT_FAILURE);

 }

 /* ---- If in verbose mode tell that we remove the alarm handler ---- */
 if (verbose) fprintf(stderr, "Remove the alarm handler!\n");

 /* ---- Remove previously set alarm ---- */
 alarm(0);

 /* ---- In verbose mode give an hexdump ---- */
 if (dump) HexDumpBuffer(*buffer);

}

void WriteAddInfo(char buffer[BUFFSIZE])
{
 char persInfo[PERSINFOLEN + 1];
 int i,j;
 bool hasValidOwner = false;

 /* ---- Write ADDINFO - Header ---- */
 printf("<ADDINFO>\n");

 /* ---- Check for valid owner-buffer ---- */
 hasValidOwner = false;
 for (j = 0x0C; j < 0x20; j++)
  if (buffer[j] != (char) 0xff) hasValidOwner = true;

 /* ---- if there is a valid owner entry print it ---- */
 if (hasValidOwner) {
  
  /* ----- Copy the personal info section into a C-string ---- */
  for (i = 0; i <= PERSINFOLEN; i++) persInfo[i] = buffer[i + 0x0C];
  persInfo[PERSINFOLEN + 1] = '\0';

  /* ---- Write the personal info ---- */
  printf(" <PERSINFO>%s</PERSINFO>\n", persInfo);  

 }

 /* ---- Write total number of dives (Unknown: buffer[0x0] belongs to TOTALDIVES?) ---- */
 printf(" <TOTALDIVES>%d</TOTALDIVES>\n", 
        256 * (int) ((unsigned char) buffer[0x01]) + 
        (int) ((unsigned char) buffer[0x02]));

 /* ---- Write the serial number ---- */
 printf(" <SERIALID>%2.2x.%2.2x.%2.2x</SERIALID>\n",
        (unsigned char) buffer[0xf4],
        (unsigned char) buffer[0xf5],
        (unsigned char) buffer[0xf6]);

 /* ---- Write the corresponding ADDINFO - Footer ---- */
 printf("</ADDINFO>\n");

}

int SearchStartOffset(char buffer[BUFFSIZE])
{
 bool found;
 int i = DATASTART;
 int j;

 /* ---- Search from 0x100 to 0x8ff for 0x82 ---- */
 found = false;
 while (i <= DATASTOP) {

  /* ---- End of profile data? ---- */
  if (buffer[i] == (char) 0x82) { 
  
   found = true; 
   break;

  }
 
  /* --- Increment index (Check next byte) ---- */
  i++;

 }

 /* ---- End marker really found? ---- */
 if (!found) {

  /* ---- Report the error and quit ---- */ 
  fprintf(stderr, "Can't find end-of-dives marker\n");
  fprintf(stderr, "Illegal profile!\n");
  exit(EXIT_FAILURE);

 }

 /* ---- If verbose that report end-of-dives ---- */
 if (verbose) fprintf(stderr, "Found end-of-dives marker at 0x%8.8x\n", i);

 /* ---- Search incrementally for end-of-next-chunk ---- */
 found = false;
 j = i; 
 while(j <= (DATASIZE + i)) {
 
  /* ---- Do a wrap in range 0x100 (DATASTART) - 0x8ff (DATASTOP) ---- */
  if (buffer[WRAP(j)] == (char) 0x80) {

   /* ---- Found end of oldest dive -> terminate loop ---- */
   found = true;
   break;

  }
 
  /* ---- Increment the index ---- */
  j++;

 } 

 /* ---- Check if oldest dive was not found ---- */
 if (!found) {

  /* ---- Report the error and quit ---- */ 
  fprintf(stderr, "Can't find oldest dive\n");
  fprintf(stderr, "Illegal profile!\n");
  exit(EXIT_FAILURE);

 }

 /* ---- Oldest dive start at position of 0x80 +3 ---- */
 return (WRAP(j + 3));

}

bool CheckDiveComplete(char buffer[BUFFSIZE], int start)
{
 int j;

 /* ---- Do a wrap in range 0x100 (DATASTART) - 0x8ff (DATASTOP) ---- */
 for (j = start; j <= DATASIZE + start + 1; j++) {
 
  /* ---- End of profile found ---- */
  if (buffer[WRAP(j)] == (char) 0x82) {

   /* ---- This is an incomplete dive entry ---- */
   return false;

  }

  /* ---- End of dive marker found ---- */
  if (buffer[WRAP(j)] == (char) 0x80) {

   /* ---- Found end of actual dive -> return true ---- */
   return true;

  }

 }

 /* ---- Neither 0x80 nor 0x82 found -> Illegal profile ---- */
 return false;

}

bool IsFirstDive(char buffer[BUFFSIZE], int start)
{
 /* ---- Check whether this dive is the first one of a group ---- */
 return (buffer[WRAP(start + 0x02)] == 1);

}

int WriteDive(char buffer[BUFFSIZE], int start)
{
 int i,j;
 int bindex;
 int depth = 0;
 bool isLast = false;
 bool wasSpecial = false;

 /* ---- Write the header ---- */
 printf(" <DIVE>\n");
 printf("  <!-- Dive number %d in this group -->\n", buffer[WRAP(start + 0x02)]);

 /* ---- Check year ---- */
 if ((unsigned int)buffer[WRAP(start + 0x06)] % 0xFF > 90) {

  /* ---- Print date of dive (Year = 19xx)---- */ 
  printf("  <DATE><YEAR>19%2.2x</YEAR><MONTH>%2.2x</MONTH><DAY>%2.2x</DAY></DATE>\n", 
         (unsigned int)buffer[WRAP(start + 0x06)] % 0xFF,
         (unsigned int)buffer[WRAP(start + 0x07)] % 0xFF,
         (unsigned int)buffer[WRAP(start + 0x08)] % 0xFF);

 } else {

  /* ---- Print date of dive (Year = 20xx) ---- */ 
  printf("  <DATE><YEAR>20%2.2x</YEAR><MONTH>%2.2x</MONTH><DAY>%2.2x</DAY></DATE>\n", 
         (unsigned int)buffer[WRAP(start + 0x06)] % 0xFF,
         (unsigned int)buffer[WRAP(start + 0x07)] % 0xFF,
         (unsigned int)buffer[WRAP(start + 0x08)] % 0xFF);

 }

 /* ---- Print time of dive ---- */
 printf("  <TIME><HOUR>%2.2x</HOUR><MINUTE>%2.2x</MINUTE></TIME>\n",
        buffer[WRAP(start + 0x09)],
        buffer[WRAP(start + 0x0A)]);

 /* ---- Check for repetitive dive ---- */
 if (buffer[WRAP(start + 0x02)] != 1) {

  /* ---- Print surfaceinterval ---- */
  printf("  <SURFACEINTERVAL>%d.0</SURFACEINTERVAL>\n", 
         60 * buffer[WRAP(start + 1)] + 
         (int) buffer[WRAP(start)]);

 } else {

  /* ---- surfaceinterval == infinity ---- */
  printf("  <SURFACEINTERVAL>infinity</SURFACEINTERVAL>\n");

 }

 /* ---- Calculate end of profile-samples ---- */
 i = 0;
 while ((buffer[WRAP(i + start + 0x0b)] != (char) 0x80) && 
        (i <= DATASIZE)) i++; 

 /* ---- Check for corrupted profile ---- */
 if (i > DATASIZE) {

  /* ---- Report the error and quit ---- */ 
  fprintf(stderr, "Can't find end of dive marker!\n");
  fprintf(stderr, "Illegal profile!\n");
  exit(EXIT_FAILURE);

 } 

 /* ---- Calculate pointer to data beyond profile samples ---- */
 bindex = WRAP(i + start + 0x0b + 1);
 
 /* ---- Print temperature ---- */
 printf("  <TEMPERATURE>%d.0</TEMPERATURE>\n", buffer[bindex] - 40);

 /* ---- Density of salt water (im kg/m^3) ---- */
 printf("  <DENSITY>1030.000</DENSITY>\n");

 /* ---- Print the altitude of the dive place ---- */
 printf("  <ALTITUDE>");
 switch (buffer[WRAP(start + 0x04)] >> 5) {
  case 0: 
   printf("700.0");  
   break;
  case 1: 
   printf("1500.0"); 
   break;
  case 2: 
   printf("2400.0"); 
   break;
  default:
   printf("Unknown");
 }
 printf("</ALTITUDE>\n");

 /* ---- Insert a mixes section. Use 'Air' only ---- */
 printf("  <GASES>\n");
 printf("   <MIX>\n");
 printf("    <MIXNAME>RealAir</MIXNAME>\n");
 if (mode == EONMode)
  printf("    <TANK><PSTART>%d.0</PSTART><PEND>%d.0</PEND></TANK>\n", 
         2 * buffer[WRAP(start + 0x05)] * BAR2PA, 
         2 * buffer[WRAP(bindex + 0x01)] * BAR2PA);
 printf("    <O2>0.21</O2>\n"); 
 printf("    <N2>0.79</N2>\n");
 printf("    <HE>0.00</HE>\n");
 printf("   </MIX>\n");
 printf("  </GASES>\n");

 /* ---- Tag for the delta - mode ---- */
 printf("  <DELTAMODE/>\n");

 /* ---- Write the samples---- */
 printf("  <SAMPLES>\n");

 /* ---- Print sample interval ---- */
 printf("   <DELTA>%d.0</DELTA>\n", buffer[WRAP(start + 0x03)]);

 /* ---- Switch to air ---- */
 printf("   <SWITCH>RealAir</SWITCH>\n");
 printf("   ");
 i = 0; j = 0; depth = 0;
 while (buffer[WRAP(i + start + 0x0b)] != (char) 0x80) {

  /* ---- Check if "/n" should be printed ---- */
  if (((j % COLUMNS) == 0) && (j > 0))printf("\n   ");

  /* ---- Check if the actual sample is the last one ---- */
  isLast = (buffer[WRAP(i + start + 0x0b + 1)] == (char) 0x80);

  /* ---- Check if surface was reached ---- */
  if (buffer[WRAP(i + start + 0x0b)] == (char) 0x7d) {

   /* ---- Check if we need a linefeed ---- */
   if (((j % COLUMNS) != 0) && ( !wasSpecial)) printf("\n   ");

   /* ---- Tell that the surface was reached ---- */
   if (isLast) 
    printf("<ALARM>SURFACE</ALARM>\n");
   else {

    /* ---- Reset column-counter and write marker ---- */
    j = -1;
    printf("<ALARM>SURFACE</ALARM>\n   ");
   }  

   /* ---- Set flag that we wrote a special marker ---- */
   wasSpecial = true;

  } else if (buffer[WRAP(i + start + 0x0b)] == (char) 0x7e) {

   /* ---- Check if we need a linefeed ---- */
   if (((j % COLUMNS) != 0) && (!wasSpecial)) printf("\n   ");

   /* ---- The nodecompression limit was left ---- */
   if (isLast) 
    printf("<ALARM>DECO</ALARM>\n");
   else { 
    
    /* ---- Reset column-counter and write marker ---- */
    j = -1;
    printf("<ALARM>DECO</ALARM>\n   ");

   }

   /* ---- Set flag that we wrote a special marker ---- */
   wasSpecial = true;

  } else if (buffer[WRAP(i + start + 0x0b)] == (char) 0x7f) {

   /* ---- Check if we need a linefeed ---- */
   if (((j % COLUMNS) != 0) && (!wasSpecial)) printf("\n   ");

   /* ---- An serious error condition occurred (Maybe deco missed?) ---- */
   if (isLast) 
    printf("<ALARM>ERROR</ALARM>\n");
   else { 
    
    /* ---- Reset column-counter and write marker ---- */
    j = -1;
    printf("<ALARM>ERROR</ALARM>\n   ");

   }

   /* ---- Set flag that we wrote a special marker ---- */
   wasSpecial = true;

  } else if (buffer[WRAP(i + start + 0x0b)] == (char) 0x81) {  
   
   /* ---- Check if we need a linefeed ---- */
   if (((j % COLUMNS) != 0) && ( !wasSpecial)) printf("\n   ");

   /* ---- We got a 'slow' warning ---- */
   if (isLast) 
    printf("<ALARM>ASCENT</ALARM>\n");
   else {

    /* ---- Reset column-counter and write marker ---- */
    j = -1;
    printf("<ALARM>ASCENT</ALARM>\n   ");

   }

   /* ---- Set flag that we wrote a special marker ---- */
   wasSpecial = true;

  } else {

   /* ---- Calculate the new depth ---- */
   depth += (signed char) buffer[WRAP(i + start + 0x0b)];

   /* ---- Sometimes EON reports negative depth's => correct this ---- */
   if (depth < 0) {

    /* ---- The depth can't be negative ---- */
    depth = 0;

    /* ---- Check if we should report this ---- */
    if (verbose)
     fprintf(stderr, "Negative depth occurred. I have fixed the problem.\n");

   }

   /* ---- Print the depth ---- */
   if (isLast)
    printf("<D>%6.2f</D>\n", depth * (double) FEET2METER);

   else
    printf("<D>%6.2f</D>", depth * (double) FEET2METER);

   /* ---- We wrote a normal depth sample ---- */
   wasSpecial = false;
 
  }

  i++; 
  j++;
 
 }
 printf("  </SAMPLES>\n");

 /* ---- Print the footer ---- */
 printf(" </DIVE>\n");

 /* ---- Return start of next dive ---- */
 return (WRAP(bindex + 0x02));

}

void WriteDiveInfo(char buffer[BUFFSIZE])
{
 int start= -1;
 bool complete;

 /* ---- Search the offset of the first dive (Always complete!) ---- */
 start = SearchStartOffset(buffer);

 /* ---- Report the start index ---- */
 if (verbose) fprintf(stderr, "Start of oldest dive at 0x%8.8x\n",start);

 /* ---- Print the header of a repetitive group ---- */
 printf("<REPGROUP>\n");

 /* ---- Check if we have a group of repetitive dives ---- */
 if (!CheckDiveComplete(buffer, start)) {

  /* ---- Give a warning ---- */
  fprintf(stderr, "This is an invalid profile! No dives available!");

  /* ---- Write end-of marker and return ---- */
  printf("</REPGROUP>\n");
  return;

 }

 /* ---- Write all dives ---- */
 while (1) {
 
  /* ---- Write the dive and use the offset for the next ---- */
  start = WriteDive(buffer, start);

  /* ---- Check next dive if eof-profile or repetitive group is reached ---- */
  complete = CheckDiveComplete(buffer, start);
 
  /* ---- Is the another repetitive group? ---- */
  if (IsFirstDive(buffer, start) && complete && !flatten) {

   /* ---- Print and and beginning of repetitive group ---- */
   printf("</REPGROUP>\n");  
   printf("<REPGROUP>\n"); 

  }

  /* ---- Check if end of list of dives reached ---- */
  if (!complete) {

   /* ---- Close the last repetitive and terminate loop ---- */
   printf("</REPGROUP>\n");  
   break;

  }

 }

}

int main(int argc, char* argv[])
{
 static char buffer[BUFFSIZE];
#ifdef HAVE_GETOPTLONG
 int optindex = (int) - 1;
 bool checksum = false; /* Calc no checksum by default */
 struct option longargdesc[] = {
                                {"verbose", no_argument, NULL, (int)'v'},
                                {"help", no_argument, NULL, (int)'h'},
                                {"dump", no_argument, NULL, (int)'p'},
                                {"flatten", no_argument, NULL, (int)'f'},
                                {"checksum", no_argument, NULL, (int)'c'},
                                {0,0,NULL,0}
                               };
#endif
 int key = (int) -1;
 time_t akttime;
 char timestr[MAXTIMELEN + 1] = ""; /* Timestring */
 size_t time_res; /* Result of strftime */
 char domainname[MAXDOMAINNAMELEN + 1] = ""; /* Buffer for getdomainname */
 char *log_name; /* Pointer to buffer with login name */                        
 char *argPtr; /* Pointer to name of this program */
 int i; /* Universal counter */
 unsigned char chksum; /* Checksum for buffer */

 /* ---- Get name of program ---- */
 if ((argPtr = strrchr(argv[0], '/')) != (char *) NULL) 
  argPtr++;
 else
  argPtr = argv[0];

 /* ---- Check for EON - Mode ---- */
 if (strcmp(argPtr, EONName) == 0)
  mode = EONMode;

 /* ---- Check for Solution Alpha mode ---- */
 if (strcmp(argPtr, SolName) == 0)
  mode = SolMode;
   
#ifndef NDEBUG
 if (mode == EONMode) fprintf(stderr, "Set to EONMode\n");
 if (mode == SolMode) fprintf(stderr, "Set to SolMode\n");
#endif

 /* ---- Parse all options ---- */
#ifdef HAVE_GETOPTLONG
 while ((key = getopt_long(argc, 
                           argv, 
                           "vhpfc", 
                           longargdesc, 
                           &optindex)) != EOF) {
#else
 while ((key = getopt(argc, argv, "vhpfc")) != EOF) {
#endif
 
  /* ---- Check for specific option ---- */
  switch (key) {
 
   case 'v':
            /* ---- Set verbose mode ---- */
            verbose = true;
            break; 
   case 'p':
            /* ---- Set dump mode ---- */
            dump = true;
            break; 
   case 'f':
            /* ---- Set flatten mode ---- */
            flatten = true;
            break; 
   case 'c':
            /* ---- Set checksum mode ---- */
            checksum = true;
            break; 
   case 'h':
            /* ---- Print the help message ---- */
            PrintHelp(argv[0]);
            break;
   case '?':

#ifdef HAVE_GETOPTLONG
            printf("Try `%s --help' for more information\n", argv[0]);
#else
            printf("Try `%s -h' for more information\n", argv[0]);
#endif
            exit(EXIT_FAILURE);

   default:
            printf("Error while parsing options!\n");
            exit(EXIT_FAILURE);

  }

 }

 /* ---- Read the buffer from stdin ---- */
 ReadBuffer(&buffer);

 /* ---- Check for checksum mode ---- */
 if (checksum) {
 
  /* ---- Calculate the checksum ---- */
  chksum = 0x00;
  for(i=0; i < BUFFSIZE - 1; ++i) chksum += buffer[i];

  /* ---- Check if checksum is correct and give correct message ---- */
  if ((unsigned char) buffer[BUFFSIZE - 1] != chksum)
   fprintf(stderr, "Warning! Checksum of data is not correct!\n");
  else
   if (verbose) fprintf(stderr, "Checksum of data is correct!\n");

 }

 /* ---- Print the Header ---- */
 printf("<PROFILE UDCF=\"1\">\n");

 /* ---- Write some header information ---- */
 if (mode == EONMode) printf("<!-- This file was generated by 'eonconvert' -->\n");
 if (mode == SolMode) printf("<!-- This file was generated by 'solaconvert' -->\n");

 /* ---- Get system-time ---- */
 akttime = time((time_t *) NULL);
 time_res = strftime(timestr,
                     MAXTIMELEN,
                     "%a %b %d %H:%M:%S %Z %Y",
                     localtime((time_t *) &akttime)
                    );
 
 /* ---- Get login name ---- */
 log_name = getenv("LOGNAME");

 /* ---- Check for valid login name and hostname ---- */
 if (log_name != (char *) NULL) {
 
  /* ---- Get the FQN hostname ---- */
  if(getFQDomain(domainname,MAXDOMAINNAMELEN)) {
 
   /* ---- Write login/hostname/domainname ---- */
   printf("<!-- Converted for: %s@%s -->\n",log_name,domainname);
 
  }
  else {
 
   /* ---- Write login/hostname ---- */
   printf("<!-- Converted for: %s -->\n",log_name);
 
  }
 
 }                                               

 /* ---- Insert conversion time in header ---- */
 printf("<!-- Date: %s -->\n", timestr);

 /* ---- We will write a units in metric system ---- */
 printf("<UNITS>Metric</UNITS>\n");

 /* ---- Check for EONMode ---- */
 if ( mode == EONMode) {

  /* ---- Print EON - device info ---- */
  printf("<DEVICE>\n");
  printf(" <VENDOR>Suunto</VENDOR>\n");
  printf(" <MODEL>EON</MODEL>\n");
  printf(" <VERSION>1</VERSION>\n");
  printf("</DEVICE>\n");

 } 

 /* ---- Check for Solution Alpha mode ---- */
 if ( mode == SolMode) {

  /* ---- Print Sol - device info ---- */
  printf("<DEVICE>\n");
  printf(" <VENDOR>Suunto</VENDOR>\n");
  printf(" <MODEL>Solution Alpha</MODEL>\n");
  printf(" <VERSION>1</VERSION>\n");
  printf("</DEVICE>\n");

 }

 /* ---- Write the additional info ---- */
 WriteAddInfo(buffer);
 
 /* ---- Write all dive information ---- */
 WriteDiveInfo(buffer);

 /* ---- Write the footer ---- */
 printf("</PROFILE>\n\n");

 /* ---- Everything is fine ---- */
 exit(EXIT_SUCCESS); 

}

