#include <assert.h>
#include <getopt.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>

#include "eon.h"
#include "bool.h"
#include "config.h"

/* N + 20 Bytes = Set personal info                      */
/* T + 1 Byte (#20, #30 oder #60) set sample-interval    */
/* P => 2304 ($900) transfer profile                     */
/* See http://www.markus-fritze.de/suunto/ for more info */

/* ---- Define an illegal argument for eonArg ---- */
#define ILLARG Illegal
static const char* Illegal = "Illegal";

/* ---- POSIX compliant source ---- */
#define _POSIX_SOURCE 1   

#define DEFDEVICE "/dev/ttyS1"
#define LOCKFILEPROTO "/var/lock/LCK..%s"

#define MAXEONNAMELEN 20 /* Length of personal info entry in an EON */

static struct termios oldtermbuf; /* Old device status */

#ifndef NDEBUG
bool verbose = true; /* == true iff verbose mode is selected */
#else 
bool verbose = false;
#endif 

/* ---- Name of lockfile and device ---- */
static char devicename[FILENAME_MAX] = DEFDEVICE;
static char lockfile[2 * FILENAME_MAX] = "";

/* ---- Filehandle of device and lockfile ---- */
int device = -1; 
int lockfd = -1;

/* ---- Number of ms to set RTS to high ---- */
static const long RTSWait = 200;

/* ---- Number of ms to wait after awaking the EON - Interface ---- */ 
static const long LittleWait = 10;

/* ---- Number of ms to wait before reading the profile buffer ---- */
static const long PreReadWait = 30;

/* ---- Number of ms to wait after creating the lockfile ---- */
static const long LongWait = 2000;

/* ---- Suspend execution for 'msec' sec * 10^-3 ---- */ 
void MWait(long msec)
{
 struct timespec ts;

 /* ---- Init timespec ---- */
 ts.tv_sec = (time_t) msec / 1000;
 ts.tv_nsec = (msec % 1000 ) * 1000 * 1000; 

 /* ---- Wait 'msec' sec*10^-3 ---- */
 (void) nanosleep(&ts, (struct timespec *) NULL);

}

/* ---- Create a lockfile for serial device ---- */
void CreateLockfile()
{
 char mypid[32];
 char* slashptr;
 struct stat statbuf;
 mode_t oldmask;

 /* ---- Get name of device without any path information ---- */
 if ((slashptr = strrchr(devicename, '/')) != (char*) NULL) {

  /* ---- Check for valid string after last '/' in 'devicename' ---- */
  if (*(slashptr + 1) != '\0') {

   /* ---- Create the lockfilename (does contain '/') ---- */
   sprintf(lockfile, LOCKFILEPROTO, slashptr + 1);

  } else {

  /* ---- Give an error message and terminate ---- */
  fprintf(stderr, "Empty devicename in %s after last '/'\n", devicename);
  exit(EXIT_FAILURE);

  }

 } else {

  /* ---- Create the lockfilename (doesn't contain '/') ---- */
  sprintf(lockfile, LOCKFILEPROTO, devicename);

 }

 /* ---- Check if lockfile already exists ---- */
 if (stat(lockfile, &statbuf) == 0) {

  /* ---- Give an error message and terminate ---- */
  fprintf(stderr, "Device %s already locked\n", devicename);
  exit(EXIT_FAILURE);

 } 

 /* ---- Check whether we should report this ---- */
 if (verbose) {

  /* ---- Print a message ---- */
  fprintf(stderr, "Info: Establish lock file %s\n", lockfile);

 }

 /* ---- Open the lockfile ---- */
 oldmask = umask(0);
 if ((lockfd = open(lockfile,
                    O_RDWR | O_CREAT, 
                    S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0) {

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't open %s as lockfile\n", lockfile);
  fprintf(stderr, "Have to quit\n"); 
  exit(EXIT_FAILURE);

 }
 (void) umask(oldmask);
 
 /* ---- Write the pid of this process into the lockfile ---- */
 mypid[0] = '\0';
 sprintf(mypid, "%ld", (long) getpid());
 if (write(lockfd, mypid, strlen(mypid)) != (ssize_t) strlen(mypid)) {

  /* ---- Can't write lockfile correctly! Better we quit ---- */
  fprintf(stderr, "Can't write lockfile correctly. Have to quit\n");
  exit(EXIT_FAILURE);

 }  

}

/* ---- Restore status to old settings ---- */
void RestoreDevice(int device)
{
 /* ---- Check if device were opened ---- */
 if (device > 0) {

  /* ---- Check whether we should report this ---- */
  if (verbose) {

   /* ---- Print a message ---- */
   fprintf(stderr, "Info: Restore device '%s' to old settings\n", devicename);

  }

  /* ---- Set the device to old mode ---- */
  if (tcsetattr(device, TCSADRAIN, &oldtermbuf) < 0) {

   /* ---- Give an error message ---- */
   fprintf(stderr, "Can't restore modus of %s\n", devicename);
   exit(EXIT_FAILURE);

  }

  /* ---- Device is closed ---- */
  device = -1;

 }

}

/* ---- Handle signals ---- */
void CleanUp()
{
 /* ---- Finish program ---- */
 exit(0);

}

/* ---- Handle cleanup when exit() was called ---- */
void DoExit()
{
 int erg;

 /* ---- Restore old term modus ---- */
 RestoreDevice(device);

 /* ---- Check whether we should report RTS is set to 'high' ---- */
 if (verbose) {

  /* ---- Print a message ---- */
  fprintf(stderr, "Info: Set RTS to high\n");

 }

 /* ---- Set RTS high  ---- */
 (void) ioctl(device, TIOCMGET, &erg);
 erg &= ~TIOCM_RTS; 
 (void) ioctl(device, TIOCMSET, &erg);

 /* ---- Check for opened device ---- */
 if (device > 0) {

  /* ---- Check for verbose mode ---- */
  if (verbose) {

   /* ---- Print a message ---- */
   fprintf(stderr, "Info: Closing the device (%s)\n", devicename);

  }

  /* ---- Close the device ---- */
  (void) close(device);

 }

 /* ---- Check if we can close and unlink the file ---- */
 if (lockfd > 0) {
 
  /* ---- Check for verbose mode ---- */
  if (verbose) {

   /* ---- Print a message ---- */
   fprintf(stderr, "Info: Close and unlink lock file %s\n", lockfile);

  }

  /* ---- Close the lockfile ---- */
  (void) close(lockfd);
  lockfd = -1;

  if (unlink(lockfile) < 0) {

   /* ---- Give a warning ---- */
   fprintf(stderr, "Can't unlink %s\n", lockfile);
   fprintf(stderr, "Delete the lockfile (%s) manually\n", lockfile); 

  } 

 }

}

/* ---- Wake up EON ---- */
void AlohaEON(int device)
{
 int erg;

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print a message ---- */
  fprintf(stderr, 
          "Info: Wake up lazy computer (Set RTS to high and reset to low in %ld ms)\n",
          RTSWait);

 }

 /* ---- Set RTS high  ---- */
 (void) ioctl(device, TIOCMGET, &erg);
 erg |= TIOCM_RTS; 
 (void) ioctl(device, TIOCMSET, &erg);

 /* ---- Wait a little time ---- */
 MWait(RTSWait);

 /* ---- Set RTS low  ---- */
 (void) ioctl(device, TIOCMGET, &erg);
 erg &= ~TIOCM_RTS; 
 (void) ioctl(device, TIOCMSET, &erg);

 /* ---- Wait a little additional time ---- */
 MWait(LittleWait);

}

/* ---- Print Help-Message ---- */
void PrintHelp(const char* prgname)
{
#ifdef HAVE_GETOPTLONG
 /* ---- Print help message ---- */
 fprintf(stderr, "Usage: %s [OPTION(s)]\n", prgname);
 if (verbose) 
  fprintf(stderr, "$Id: eondump.c,v 1.20 2001/09/30 13:59:50 streit Exp $\n");
 fprintf(stderr, "-h --help                 Print a help text\n");
 fprintf(stderr, "-v --verbose              Explain what is done\n");
 fprintf(stderr, "-d --device filename      Specify seriell device\n");
 fprintf(stderr, "-r --read[=filename | =-] Read profile from EON\n");
#ifdef DANGER
 fprintf(stderr, "-t --time dt              Set sample interval to 'dt'\n");
 fprintf(stderr, "-s --set info             Set personal info\n");
#endif
 fprintf(stderr, "\n");
#else
 /* ---- Print help message ---- */
 fprintf(stderr, "Usage: %s [OPTION(s)]\n", prgname);
 if (verbose) 
  fprintf(stderr, "$Id: eondump.c,v 1.20 2001/09/30 13:59:50 streit Exp $\n");
 fprintf(stderr, "-h                        Print a help text\n");
 fprintf(stderr, "-v                        Explain what is done\n");
 fprintf(stderr, "-d filename               Specify seriell device\n");
 fprintf(stderr, "-r [=filename | =-]       Read profile from EON\n");
#ifdef DANGER
 fprintf(stderr, "-t dt                     Set sample interval to 'dt'\n");
 fprintf(stderr, "-s info                   Set personal info\n");
#endif
 fprintf(stderr, "\n");
#endif

 /* ---- Terminate program successfully ---- */
 exit(EXIT_SUCCESS); 

}

/* ---- Read profile from EON ---- */
void ReadProfile(const char* outputName)
{
 ssize_t nread;
 char eonbuf[BUFFSIZE]; /* Buffer for EON-Data */
 int outfd; 
 int i,j;
 bool hasValidOwner = false;
 mode_t oldmask;

 /* ---- Check if we have to use stdout ---- */
 if (strcmp(outputName, "-") != 0) {

  /* ---- Open outputfile ---- */
  oldmask = umask(0);
  if ((outfd = open(outputName, 
       O_WRONLY | O_CREAT,
       S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0) {

   /* ---- Give an error message ---- */
   fprintf(stderr, "Can't open %s for output\n", outputName);
   fprintf(stderr, "Have to quit\n"); 
   exit(EXIT_FAILURE);

  }
 (void) umask(oldmask); 

  /* ---- Check for verbose mode ---- */
  if (verbose) {

   /* ---- Print the message ---- */
   fprintf(stderr, 
           "Info: Use %s as output file for profile data\n", 
           outputName);

  }
  
 } else {

  /* ---- We use stdout ---- */
  outfd = STDOUT_FILENO;

  /* ---- Check for verbose mode ---- */
  if (verbose) {

   /* ---- Print the message ---- */
   fprintf(stderr, "Info: Use 'stdout' for profile data\n");

  }

 }

 /* ---- Wake the lazy dive-computer ---- */
 AlohaEON(device);

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, "Info: Start transfer of profile\n");

 }

 /* ---- Tell EON to start data transfer ---- */
 if (write(device, "P", 1) != 1) { 

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't tell EON to start data transfer\n");

  /* ---- Terminate program ---- */
  exit(EXIT_FAILURE);

 }

 /* ---- Wait before reading the profile buffer ---- */
 MWait(PreReadWait);

 /* ---- Read whole EON buffer ---- */
 i = 0;
 while (i < BUFFSIZE) {

  /* ---- Read one byte from EON ---- */
  nread = read(device, &eonbuf[i++], 1);
 
  /* ---- Check for verbose mode ---- */
  if (verbose) {

   /* ---- Show progress ---- */
   fprintf(stderr, ".");

  }
  
 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print newline ---- */
  fprintf(stderr, "\n");

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, "Info: Profile read successfully\n");

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print some additional info ---- */
  fprintf(stderr, "Info: Write profile data to file %s\n", outputName);

 }

 /* ---- Write to output file ---- */
 if (write(outfd, eonbuf, BUFFSIZE) != BUFFSIZE) {

  /* ---- Tell that outputfile isn't complete ---- */
  fprintf(stderr, "Warning: Outputfile isn't complete\n");
  
 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print some additional info ---- */
  fprintf(stderr, "\n");
  fprintf(stderr, "Time-interval: %d\n", (int) eonbuf[9]);

  /* ---- Check for valid owner-buffer ---- */
  hasValidOwner = false;
  for (j = 0x0C; j < 0x20; j++)
  if (eonbuf[j] != (char) 0xff) hasValidOwner = true;

  /* ---- Check for a valid owner entry ---- */
  if (hasValidOwner) {

   /* ---- Terminate string ---- */
   eonbuf[0x20] = '\0'; 
   fprintf(stderr, "Owner: \"%s\"\n", &eonbuf[0x0C]);
  
  }

  /* ---- Print dive-computer identification ---- */
  fprintf(stderr, "Identification number: %2.2x.%2.2x.%2.2x\n", 
         (unsigned char) eonbuf[0xf4], 
         (unsigned char) eonbuf[0xf5], 
         (unsigned char) eonbuf[0xf6]);
  fprintf(stderr, "\n");

 }

 /* ---- Close outputfile (if != stdout) ---- */
 if (outfd !=  STDOUT_FILENO) (void) close(outfd);

}


void SetName(const char* personalName)
{
 char name[MAXEONNAMELEN + 1] = "";
 size_t i,perslen;

 /* ---- Test for illegal arg ---- */
 if (strcmp(personalName, ILLARG) == 0) {

  /* ---- Print an error message and terminate ---- */
  fprintf(stderr, "There occured an error while setting the personal name.\n");
  fprintf(stderr, "The name is not set correctly\n");
  exit(EXIT_FAILURE);

 } 

 /* ---- Calc length of personalName and check for MAXEONNAMELEN ---- */
 if ((perslen = strlen(personalName)) > MAXEONNAMELEN) {

  /* ---- Give a warning ---- */
  fprintf(stderr, 
          "Warning: Personal name is too long. Only use %ld characters.\n",
          (long) MAXEONNAMELEN);

 } 

 /* ---- Copy maximal MAXEONNAMELEN Bytes in 'name' ---- */
 name[MAXEONNAMELEN] = '\0';
 for (i=0; i< MAXEONNAMELEN; i++) {

  /* ---- Check if end of string 'personalName' is already reached ---- */
  if (i < perslen) {

   /* ---- Copy all characters of personalName without '\0' ---- */ 
   name[i] = personalName[i];
 
  } else {

   /* ----- Simply fill in ' ' ---- */
   name[i] = ' '; 

  }

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, 
          "Info: Trying to set personal name to '%s'\n", 
          name);

 }

 /* ---- Wake the lazy dive-computer ---- */
 AlohaEON(device);

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, "Info: Start transfer of name '%s'\n", name);

 }

 /* ---- Tell EON to start data transfer ---- */
 if (write(device, "N", 1) != 1) { 

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't tell EON to set name.\n");

  /* ---- Terminate program ---- */
  exit(EXIT_FAILURE);

 }

 /* ---- Set personal name ---- */
 for (i = 0; i < MAXEONNAMELEN; i++) {

  if (write(device, &name[i], 1) != 1) {

   /* ---- Give an error message ---- */
   fprintf(stderr, 
           "Name could not set sucessfully. (Error while writing byte %ld)\n",
           (long int) i + 1);

   /* ---- Terminate program ---- */
   exit(EXIT_FAILURE);

  } 

  /* ---- Wait a short time ---- */
  MWait(LittleWait);

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, "Info: Name set successfully to '%s'\n", name);

 }

}

void SetTime(const int interval)
{
 char chint;
 ssize_t nwritten;

 /* ---- Check for valid values ---- */ 
 if ((interval != 20) && (interval != 30) && (interval != 60)) {

  /* ---- Print an error message and terminate ---- */
  fprintf(stderr, "Only 20, 30 or 60 sec are valid time intervals.\n");
  fprintf(stderr, "Other intervals can destroy your EON.\n");
  exit(EXIT_FAILURE);

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the time interval ---- */
  fprintf(stderr, "Info: Set time inteval to %d sec\n", interval);

 }

 /* ---- Wake the lazy dive-computer ---- */
 AlohaEON(device);

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print start message ---- */
  fprintf(stderr, "Info: Start transfer of time interval\n");

 }

 /* ---- Tell EON to start data transfer ---- */
 if (write(device, "T", 1) != 1) { 

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't tell EON to set the time interval.\n");

  /* ---- Terminate program ---- */
  exit(EXIT_FAILURE);

 }

 /* ---- Cast time into and copy it into EON ---- */
 chint = (char) interval; 
 if ((nwritten = write(device, 
                       &chint, 
                       sizeof(char))) != (ssize_t) sizeof(char)) {

  /* ---- Give an error message ---- */
  fprintf(stderr, 
          "Name could not set sucessfully. (%ld Bytes written instead %ld)\n",
          (long) nwritten, (long) MAXEONNAMELEN );

  /* ---- Terminate program ---- */
  exit(EXIT_FAILURE);

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print finished message ---- */
  fprintf(stderr, "Info: Time interval set successfully.\n");

 }

}

int main (int argc, char* argv[])
{
 struct termios termbuf; /* Handle for termios functions */
 int key;
 bool readEON = false;
 bool setTime = false;
 bool setInfo = false;
 const char* eonArg = ILLARG;
#ifdef HAVE_GETOPTLONG
 int optindex;
 struct option longargdesc[] = {
                                {"verbose", no_argument, NULL,      (int)'v'},
                                {"device",  required_argument, NULL,(int)'d'},
                                {"help",    no_argument, NULL,      (int)'h'},
                                {"read",    required_argument, NULL,(int)'r'},
#ifdef DANGER
                                {"time",    required_argument, NULL,(int)'t'},
                                {"set",     required_argument, NULL,(int)'s'}, 
#endif
                                {0,         0,                 NULL, 0}
                            };   
#endif

#ifdef HAVE_GETOPTLONG
 /* ---- Parse all options ---- */
 while ((key = getopt_long(argc, 
                           argv, 
#ifdef DANGER
                           "vhd:r:t:s:", 
#else
                           "vhd:r:", 
#endif
                           longargdesc, 
                           &optindex)) != EOF) {
#else 
 /* ---- Parse all options ---- */
 while ((key = getopt(argc, 
                      argv, 
#ifdef DANGER
                      "vhd:r:t:s:" 
#else
                      "vhd:r:" 
#endif
                      )) != EOF) {
#endif
 
  /* ---- Check for specific option ---- */
  switch (key) {
 
   case 'v':
            /* ---- Set verbose mode ---- */
            verbose = true;
            break; 
   case 'h':
            /* ---- Print the help message ---- */
            PrintHelp(argv[0]);
            break;
   case 'd':
            /* ---- Check for verbose mode ---- */
            if (verbose) {

             /* ---- Print a message ---- */
             fprintf(stderr, "Info: Set device to %s\n", optarg);

            }

            /* ---- Copy new device name ---- */
            strncpy(devicename, optarg, (size_t) (FILENAME_MAX - 1));
            break;
 
   case 'r':
            /* ---- We have to read profile data from EON ---- */
            readEON = true;
            eonArg = optarg;
            break;

   case 't':
            /* ---- We have to set the time interval ---- */
            setTime = true;
            eonArg = optarg;
            break;

   case 's':
            /* ---- Set personal info ---- */
            setInfo = true;
            eonArg = optarg;
            break;

   case '?':

#ifdef HAVE_GETLONG
            printf("Try `%s --help' for more information.\n", argv[0]);
#else
            printf("Try `%s -h' for more information.\n", argv[0]);
#endif
            exit(EXIT_FAILURE);

   default:
            printf("Error while parsing options.\n");
            exit(EXIT_FAILURE);

  }

 }

 /* ---- Check whether one of -r -s -t was selected ---- */
 if (!(readEON || setTime || setInfo)) {

#ifdef DANGER

  /* ---- Give an error message ---- */
  fprintf(stderr, 
          "Your have to select one option out of -r, -s or -t\n"); 
#else

  /* ---- Give an error message ---- */
  fprintf(stderr, 
          "Your have to select the option -r\n"); 
#endif

  /* ---- Terminate with an error ---- */
  exit(EXIT_FAILURE);

 }

 /* ---- Check that exactly one option was selected ---- */
 if ((readEON && setTime) || (readEON && setInfo) || (setTime && setInfo)) {

  /* ---- Give an error message and terminate ---- */
  fprintf(stderr, 
          "Your have to select exactly one option of -r, -s or -t\n"); 
  exit(EXIT_FAILURE);

 }

 /* ---- Install signal & exit - handler ---- */
 signal(SIGINT,  CleanUp);
 signal(SIGQUIT, CleanUp);
 signal(SIGTERM, CleanUp);
 (void) atexit(DoExit);

 /* ---- Create the lockfile and wait ---- */
 CreateLockfile(); MWait(LongWait);

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, "Info: Open device %s\n", devicename);

 }

 /* ---- Open serial device ---- */
 device = open(devicename, O_RDWR | O_NOCTTY );
 if (device < 0) {

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't open %s as input.\n", devicename);
  exit(EXIT_FAILURE);

 }

 /* ---- Check for verbose mode ---- */
 if (verbose) {

  /* ---- Print the message ---- */
  fprintf(stderr, "Info: Set device %s to 1200 8N2\n", devicename);

 }

 /* ---- Get device status ---- */
 if (tcgetattr(device, &oldtermbuf) < 0) {

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't get status of %s.\n", devicename);
  exit(EXIT_FAILURE);

 }

 /* ---- Set termios-struct for raw mode (Mode: 1200 8N2) ---- */
 termbuf.c_cflag = B1200 | CS8 | CLOCAL | CREAD | CSTOPB;
 termbuf.c_iflag =  IGNPAR;
 termbuf.c_oflag = 0;
 termbuf.c_lflag = 0;
 termbuf.c_cc[VTIME] = 0;
 termbuf.c_cc[VMIN] = 1;

 /* ---- Set the device to raw mode (Mode: 1200 8N2) ---- */
 (void) tcflush(device, TCIFLUSH);
 if (tcsetattr(device, TCSANOW, &termbuf) < 0) {

  /* ---- Give an error message ---- */
  fprintf(stderr, "Can't set modus of %s\n", devicename);
  exit(EXIT_FAILURE);

 }

 /* ---- Should we read EON's profile? ---- */
 if (readEON) ReadProfile(eonArg);

 /* ---- Should we set the personal name? ---- */
 if (setInfo) SetName(eonArg);

 /* ---- Should we set the time interval? ---- */
 if (setTime) SetTime(atoi(eonArg));

 /* --- Everything was fine (Call exit-handler) ---- */
 exit(EXIT_SUCCESS);

}
